import React, { useState } from 'react';
import { Accordion, Segment, Button, Form } from 'semantic-ui-react';
import { defineMessages, FormattedMessage, injectIntl } from 'react-intl';
import fieldsData from './fields-data';
import actionsData from './actions-data';
import { Icon, TextWidget, CheckboxWidget } from '@plone/volto/components';
import PropTypes from 'prop-types';

import upSVG from '@plone/volto/icons/up-key.svg';
import downSVG from '@plone/volto/icons/down-key.svg';

const messages = defineMessages({
  Form: {
    id: 'Form',
    defaultMessage: 'Form Block',
  },
  Fields: {
    id: 'Fields',
    defaultMessage: 'Fields',
  },
  Information: {
    id: 'Information',
    defaultMessage: 'Information',
  },
  insertField: {
    id: 'insertField',
    defaultMessage: 'Insert new Field',
  },
});

const FormSidebar = ({ data, block, onChangeBlock, intl, props }) => {
  const [activeAccIndex, setActiveAccIndex] = useState(0);
  const formfields = data.formfields || [];
  const formactions = data.formactions || [];
  const activeFormField = data.activeFormField || 0;

  function handleAccClick(e, titleProps) {
    const { index } = titleProps;
    const newIndex = activeAccIndex === index ? -1 : index;

    setActiveAccIndex(newIndex);
  }

  function addItem(item) {
    onChangeBlock(block, {
      ...data,
      formfields: [...formfields, item],
    });
  }

  function updateItem(item) {
    let newArray = data.formfields;
    newArray[item.index] = { ...newArray[item.index], [item.name]: item.value };
    onChangeBlock(block, {
      ...data,
      formfields: newArray,
    });
  }

  function updateAttributes(item) {
    let newArray = data.formfields;
    newArray[item.index].attributes = {
      ...newArray[item.index].attributes,
      [item.name]: item.value,
    };
    onChangeBlock(block, {
      ...data,
      formfields: newArray,
    });
  }

  function addAction(item) {
    onChangeBlock(block, {
      ...data,
      formactions: [...formactions, item],
    });
  }

  function updateAction(item) {
    let newArray = data.formactions;
    newArray[item.index] = { ...newArray[item.index], [item.name]: item.value };
    onChangeBlock(block, {
      ...data,
      formactions: newArray,
    });
  }

  function updateActionConfig(item) {
    let newArray = data.formactions;
    newArray[item.index].configuration[item.key] = {
      ...newArray[item.index].configuration[item.key],
      [item.name]: item.value,
    };
    onChangeBlock(block, {
      ...data,
      formactions: newArray,
    });
  }

  return (
    <Segment.Group raised>
      <header className="header pulled">
        <h2>
          <FormattedMessage id="Form" defaultMessage="Form Block" />
        </h2>
      </header>

      <>
        <Accordion fluid styled className="form">
          <Accordion.Title
            active={activeAccIndex === 0}
            index={0}
            onClick={handleAccClick}
          >
            Fields
            {activeAccIndex === 0 ? (
              <Icon name={upSVG} size="20px" />
            ) : (
              <Icon name={downSVG} size="20px" />
            )}
          </Accordion.Title>
          <Accordion.Content active={activeAccIndex === 0}>
            <>
              {fieldsData &&
                fieldsData.map((item, index) => (
                  <Button
                    key={index}
                    title={intl.formatMessage(messages.insertField)}
                    aria-label={intl.formatMessage(messages.insertField)}
                    onClick={() => {
                      addItem({
                        id: item.id,
                        title: item.title,
                        label: item.label,
                        fieldgroup: item.fieldgroup,
                        type: item.type,
                        placeholder: item.placeholder,
                        attributes: item['attributes'],
                      });
                    }}
                  >
                    {item.title}
                  </Button>
                ))}
            </>
          </Accordion.Content>
        </Accordion>
      </>

      <>
        <Accordion fluid styled className="form">
          <Accordion.Title
            active={activeAccIndex === 1}
            index={1}
            onClick={handleAccClick}
          >
            Field-Settings
            {activeAccIndex === 1 ? (
              <Icon name={upSVG} size="20px" />
            ) : (
              <Icon name={downSVG} size="20px" />
            )}
          </Accordion.Title>
          <Accordion.Content active={activeAccIndex === 1}>
            {data.formfields && activeFormField !== null ? (
              <>
                <TextWidget
                  id="label"
                  title="Set Label"
                  required={false}
                  value={data.formfields[activeFormField].label}
                  name="label"
                  onChange={(name, value) => {
                    updateItem({
                      index: activeFormField,
                      value: value,
                      name: name,
                    });
                  }}
                />
                <TextWidget
                  id="placeholder"
                  title="Set Placeholder"
                  required={false}
                  value={data.formfields[activeFormField].placeholder}
                  name="placeholder"
                  onChange={(name, value) => {
                    updateItem({
                      index: activeFormField,
                      value: value,
                      name: name,
                    });
                  }}
                />
                <CheckboxWidget
                  id="required"
                  title="Field is Required"
                  value={
                    data.formfields[activeFormField].required
                      ? data.formfields[activeFormField].required
                      : false
                  }
                  name="required"
                  onChange={(name, value) => {
                    updateItem({
                      index: activeFormField,
                      value: value,
                      name: name,
                    });
                  }}
                />
                {Object.keys(data.formfields[activeFormField].attributes).map(
                  function (keyName, keyIndex) {
                    return (
                      <TextWidget
                        id={keyName}
                        name={keyName}
                        title={keyName}
                        value={
                          data.formfields[activeFormField].attributes[keyName]
                        }
                        onChange={(name, value) => {
                          typeof data.formfields[activeFormField].attributes[
                            keyName
                          ] === 'number'
                            ? (value = parseInt(value))
                            : (value = value);
                          updateAttributes({
                            index: activeFormField,
                            value: value,
                            name: name,
                          });
                        }}
                      />
                    );
                  },
                )}
              </>
            ) : (
              <div>No Field selected</div>
            )}
          </Accordion.Content>
        </Accordion>
      </>

      <>
        <Accordion fluid styled className="form">
          <Accordion.Title
            active={activeAccIndex === 2}
            index={2}
            onClick={handleAccClick}
          >
            Action-Settings
            {activeAccIndex === 2 ? (
              <Icon name={upSVG} size="20px" />
            ) : (
              <Icon name={downSVG} size="20px" />
            )}
          </Accordion.Title>
          <Accordion.Content active={activeAccIndex === 2}>
            <>
              {actionsData && formactions.length === 0
                ? actionsData.map((item, index) => (
                    <Button
                      key={index}
                      title={intl.formatMessage(messages.insertField)}
                      aria-label={intl.formatMessage(messages.insertField)}
                      onClick={() => {
                        addAction({
                          id: item.id,
                          title: item.title,
                          submitButtonText: item.submitButtonText,
                          configuration: item.configuration,
                        });
                      }}
                    >
                      {item.title}
                    </Button>
                  ))
                : data.formactions.map((item, index) => (
                    <>
                      <TextWidget
                        id="submitButtonText"
                        title="Submitbutton Text"
                        required={false}
                        value={data.formactions[index].submitButtonText}
                        name="submitButtonText"
                        onChange={(name, value) => {
                          updateAction({
                            index: index,
                            value: value,
                            name: name,
                          });
                        }}
                      />
                      {Object.keys(data.formactions[index].configuration).map(
                        function (keyName, keyIndex) {
                          return (
                            <Form.Input
                              id={keyName}
                              key={keyName}
                              index={keyIndex}
                              title={keyName}
                              {...item.configuration[keyName]}
                              value={
                                data.formactions[index].configuration[keyName]
                                  .value
                              }
                              onChange={(name, value) => {
                                updateActionConfig({
                                  index: index,
                                  value: value.value,
                                  name: 'value',
                                  key: keyName,
                                });
                              }}
                            />
                          );
                        },
                      )}
                    </>
                  ))}
            </>
          </Accordion.Content>
        </Accordion>
      </>
    </Segment.Group>
  );
};

FormSidebar.propTypes = {
  data: PropTypes.objectOf(PropTypes.any).isRequired,
  block: PropTypes.string.isRequired,
  onChangeBlock: PropTypes.func.isRequired,
};

export default injectIntl(FormSidebar);
